pub struct PgVarlena<T>{ /* private fields */ }Expand description
Wraps a Postgres varlena *, presenting it as if it’s a Rust type of a fixed size.
The wrapped varlena * is behind a Rust alloc::borrow:Cow which ensures that in the
common-use case of creating a PgVarlena from a Postgres-provided Datum, it’s not
possible to scribble on that Postgres-allocated memory.
Generally, PgVarlena is meant to be used in conjunction with pgx’s PostgresType derive macro
to provide transparent mapping of fixed-size Rust types as Postgres datums.
Example
use std::str::FromStr;
use pgx::prelude::*;
#[derive(Copy, Clone, PostgresType)]
#[pgvarlena_inoutfuncs]
struct MyType {
a: f32,
b: f32,
c: i64
}
impl PgVarlenaInOutFuncs for MyType {
fn input(input: &core::ffi::CStr) -> PgVarlena<Self> {
let mut iter = input.to_str().unwrap().split(',');
let (a, b, c) = (iter.next(), iter.next(), iter.next());
let mut result = PgVarlena::<MyType>::new();
result.a = f32::from_str(a.unwrap()).expect("a is not a valid f32");
result.b = f32::from_str(b.unwrap()).expect("b is not a valid f32");
result.c = i64::from_str(c.unwrap()).expect("c is not a valid i64");
result
}
fn output(&self, buffer: &mut pgx::StringInfo) {
buffer.push_str(&format!("{},{},{}", self.a, self.b, self.c));
}
}
#[pg_extern]
fn do_a_thing(mut input: PgVarlena<MyType>) -> PgVarlena<MyType> {
input.c += 99; // performs a copy-on-write
input
}Implementations
Sourceimpl<T> PgVarlena<T>
impl<T> PgVarlena<T>
Sourcepub fn new() -> Self
pub fn new() -> Self
Create a new PgVarlena representing a Rust type. The backing varlena is allocated
by Postgres and initially zero’d (using pg_sys::palloc0). Unless .into_pg() is called,
the Postgres-allocated memory will follow Rust’s drop semantics.
Example
use pgx::prelude::*;
#[derive(Copy, Clone)]
struct MyType {
a: f32,
b: f32,
c: i64
}
let mut v = PgVarlena::<MyType>::new();
v.a = 42.0;
v.b = 0.424242;
v.c = 42;Sourcepub unsafe fn from_datum(datum: Datum) -> Self
pub unsafe fn from_datum(datum: Datum) -> Self
Construct a PgVarlena from a known-to-be-non-null pg_sys::Datum. As
FromDatum for PgVarlena<T> where T: Copy + Sized is already implemented, it is unlikely
that this function will need to be called directly.
The provided datum is automatically detoasted and the returned PgVarlena will either
be considered borrowed or owned based on if detoasting actually needed to allocate memory.
If it didn’t, then we’re borrowed, otherwise we’re owned.
Safety
This function is considered unsafe as it cannot guarantee the provided pg_sys::Datum is a
valid *mut pg_sys::varlena.