Function pg_getarg_type

pub unsafe fn pg_getarg_type(fcinfo: FunctionCallInfo, num: usize) -> Oid
Safety

The provided fcinfo must be valid otherwise this function results in undefined behavior due to an out of bounds read.