pub unsafe fn pg_return_null(fcinfo: FunctionCallInfo) -> DatumExpand description
Modifies the specified fcinfo struct to indicate that its return value is null.
Examples
use pgx::pg_return_null;
use pgx::prelude::*;
fn foo(fcinfo: pg_sys::FunctionCallInfo) -> pg_sys::Datum {
return unsafe { pg_return_null(fcinfo) };
}Safety
This function is unsafe as we cannot ensure the fcinfo argument is a valid
pg_sys::FunctionCallInfo pointer. This is your responsibility.