pub unsafe fn pg_getarg_type(fcinfo: FunctionCallInfo, num: usize) -> OidExpand description
Safety
The provided fcinfo must be valid otherwise this function results in undefined behavior due
to an out of bounds read.
pub unsafe fn pg_getarg_type(fcinfo: FunctionCallInfo, num: usize) -> OidThe provided fcinfo must be valid otherwise this function results in undefined behavior due
to an out of bounds read.