pub struct PgRelation { /* private fields */ }Implementations
Sourceimpl PgRelation
impl PgRelation
Sourcepub unsafe fn from_pg(ptr: Relation) -> Self
pub unsafe fn from_pg(ptr: Relation) -> Self
Wrap a Postgres-provided pg_sys::Relation.
It is assumed that Postgres will later RelationClose() the provided relation pointer.
As such, it is not closed when this instance is dropped
Safety
This method is unsafe as we cannot ensure that this relation will later be closed by Postgres
Sourcepub unsafe fn from_pg_owned(ptr: Relation) -> Self
pub unsafe fn from_pg_owned(ptr: Relation) -> Self
Wrap a Postgres-provided pg_sys::Relation.
The provided Relation will be closed via pg_sys::RelationClose when this instance is dropped
Sourcepub unsafe fn open(oid: Oid) -> Self
pub unsafe fn open(oid: Oid) -> Self
Given a relation oid, use pg_sys::RelationIdGetRelation() to open the relation
If the specified relation oid was recently deleted, this function will panic.
Additionally, the relation is closed via pg_sys::RelationClose() when this instance is
dropped.
Safety
The caller should already have at least AccessShareLock on the relation ID, else there are nasty race conditions.
As such, this function is unsafe as we cannot guarantee that this requirement is true.
Sourcepub unsafe fn with_lock(oid: Oid, lockmode: LOCKMODE) -> Self
pub unsafe fn with_lock(oid: Oid, lockmode: LOCKMODE) -> Self
relation_open - open any relation by relation OID
If lockmode is not “NoLock”, the specified kind of lock is obtained on the relation.
An error is raised if the relation does not exist.
Safety
Using an inappropriate lockmode, such as using NoLock on a relation that has not been previously locked, may result in undefined behavior.
NB: a “relation” is anything with a pg_class entry. The caller is expected to check whether the relkind is something it can handle.
The opened relation is automatically closed via pg_sys::relation_close()
when this instance is dropped
Sourcepub unsafe fn open_with_name(relname: &str) -> Result<Self, &'static str>
pub unsafe fn open_with_name(relname: &str) -> Result<Self, &'static str>
Given a relation name, use pg_sys::to_regclass to look up its oid, and then
pg_sys::RelationIdGetRelation() to open the relation.
If the specified relation name is not found, we return an Err(&str).
If the specified relation was recently deleted, this function will panic.
Additionally, the relation is closed via pg_sys::RelationClose() when this instance is
dropped.
Safety
The caller should already have at least AccessShareLock on the relation ID, else there are nasty race conditions.
As such, this function is unsafe as we cannot guarantee that this requirement is true.
Given a relation name, use pg_sys::to_regclass to look up its oid, and then
open it with an AccessShareLock
If the specified relation name is not found, we return an Err(&str).
If the specified relation was recently deleted, this function will panic.
Additionally, the relation is closed via pg_sys::RelationClose() when this instance is
dropped.
Sourcepub fn name(&self) -> &str
pub fn name(&self) -> &str
RelationGetRelationName Returns the rel’s name.
Note that the name is only unique within the containing namespace.
Sourcepub fn namespace_oid(&self) -> Oid
pub fn namespace_oid(&self) -> Oid
RelationGetNamespace Returns the rel’s namespace OID.
Sourcepub fn namespace(&self) -> &str
pub fn namespace(&self) -> &str
What is the name of the namespace in which this relation is located?
Sourcepub fn heap_relation(&self) -> Option<PgRelation>
pub fn heap_relation(&self) -> Option<PgRelation>
If this PgRelation represents an index, return the PgRelation for the heap
relation to which it is attached
Sourcepub fn indices(&self, lockmode: LOCKMODE) -> impl Iterator<Item = PgRelation>
pub fn indices(&self, lockmode: LOCKMODE) -> impl Iterator<Item = PgRelation>
Return an iterator of indices, as PgRelations, attached to this relation
Sourcepub fn tuple_desc(&self) -> PgTupleDesc<'_>
pub fn tuple_desc(&self) -> PgTupleDesc<'_>
Returned a wrapped PgTupleDesc
The returned PgTupleDesc is tied to the lifetime of this PgRelation instance.
use pgx::{PgRelation, pg_sys};
let oid = example_pg_class_oid(42); // a valid pg_class "oid" value
let relation = unsafe { PgRelation::from_pg(pg_sys::RelationIdGetRelation(oid) ) };
let tupdesc = relation.tuple_desc();
// assert that the tuple descriptor has 12 attributes
assert_eq!(tupdesc.len(), 12);Sourcepub fn reltuples(&self) -> Option<f32>
pub fn reltuples(&self) -> Option<f32>
Number of tuples in this relation (not always up-to-date)
pub fn is_table(&self) -> bool
pub fn is_matview(&self) -> bool
pub fn is_index(&self) -> bool
pub fn is_view(&self) -> bool
pub fn is_sequence(&self) -> bool
pub fn is_composite_type(&self) -> bool
pub fn is_foreign_table(&self) -> bool
pub fn is_partitioned_table(&self) -> bool
pub fn is_toast_value(&self) -> bool
Methods from Deref<Target = PgBox<RelationData>>